/******************************************************************************
 * FILE:  L_imu_daq.c
 *
 * DESCRIPTION:
 * Data acquisition code acquires data from uNAV IMU over serial
 * port of MPC5200
 *
 *
 * Original Author: 8/31/06 Jung Soon Jang
 *
 * University of Minnesota 
 * Aerospace Engineering and Mechanics 
 * Copyright 2011 Regents of the University of Minnesota. All rights reserved.
 *
 * $Id: L_imu_daq.c 332 2011-04-14 18:43:14Z murch $
 ******************************************************************************/

#include <fcntl.h>   		/* File control definitions */
#include <errno.h>   		/* Error number definitions */
#include <termios.h> 		/* POSIX terminal control definitions */
#include <cyg/io/io.h>
#include <cyg/io/serialio.h>

#ifdef MB_ADS
#include "sensors/AirData/airDataSys.c"
#endif

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//uNAV packet length definition
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#define FULL_PACKET_LENGTH		38
#define SENSOR_PACKET_LENGTH   	51
#define GPS_PACKET_LENGTH		35
#define FULL_PACKET_SIZE        86   // scaled mode with sampling less than 100Hz


//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//prototype definition
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
int 	checksum(byte* buffer, int packet_len);
int 	open_serial(char* serial_port,int baudrate);
void 	decode_imupacket(struct imu *data, byte* buffer);
void 	decode_gpspacket(struct gps *data, byte* buffer);
extern double get_Time();
extern int sPort1, sPort0;


//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//global variables
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++



void *sensordaq(void *thread_id)
{
	int				headerOK=0, rc;
	int 			nbytes=0;
	static short    	gpsHealthCount=0;
	byte  			input_buffer[FULL_PACKET_SIZE]={0,};
	byte  			gps_buffer[GPS_PACKET_LENGTH]={0,};
	double 			toc;
	static double 		tic;
	cyg_io_handle_t console_handle;
	unsigned int	len, bytesInBuffer;
	cyg_serial_buf_info_t buff_info;

	len = sizeof (buff_info);
	if (cyg_io_lookup (SERIAL_PORT1, &console_handle) < 0)
		fprintf(stderr,"\n <IMUDAQ thread>: unable to obtain handle for %s - %s\n", SERIAL_PORT1, strerror(errno));


	pthread_mutex_lock(&mutex_sPort0);
	sprintf(&msg[3],"IMUDAQ thread created");
	write(sPort0, msg, sizeof(msg));
	pthread_mutex_unlock(&mutex_sPort0);

	/* the following four words are sent to micronav IMU to configure the output
     data packet. (1)SCALED_MODE->output data is scaled to engg. values, other
     option is raw 'voltage' output. (2)CH_BAUD->change baud rate. (3)CH_SAMP
     ->sampling rate,Hz. Reference: MNAV1000 User Manual.pdf */
	byte     	CH_BAUD_57K[11] = {0x55,0x55,0x57,0x46,0x01,0x00,0x02,0x00,0x03,0x00,0xA3};	// 57.6 kbps
	//byte     	CH_BAUD_38K[11] = {0x55,0x55,0x53,0x46,0x01,0x00,0x02,0x00,0x02,0x00,0x9E};	// 38.4 kbps
	byte  		SCALED_MODE[11] = {0x55,0x55,0x53,0x46,0x01,0x00,0x03,0x00,0x53,0x00,0xF0};	// scaled mode
	byte		CH_SAMP_50[11]  = {0x55,0x55,0x53,0x46,0x01,0x00,0x01,0x00,0x02,0x00,0x9D};	// 50 Hz
	//byte		CH_SAMP_QUIET[11]  = {0x55,0x55,0x57,0x46,0x01,0x00,0x01,0x00,0x00,0x00,0x9F};	// quiet

	// initial values
	gpspacket.err_type = gps_nolock;
	imupacket.err_type = got_invalid;

	/*********************************************************************
	 * Open and configure Serial
	 *********************************************************************/

	sPort1 = open_serial(SERIAL_PORT1,BAUDRATE_38400);

	while (nbytes != 11) {nbytes += write(sPort1,&CH_SAMP_50[nbytes], 11-nbytes);}
	nbytes = 0;
	cyg_thread_delay(100);
	while (nbytes != 11) {nbytes += write(sPort1,&SCALED_MODE[nbytes],11-nbytes);}
	nbytes = 0;
	cyg_thread_delay(100);

	while (nbytes != 11) {nbytes += write(sPort1,&CH_BAUD_57K[nbytes],11-nbytes);}
	nbytes = 0;
	close(sPort1);
	cyg_thread_delay(100);

	sPort1 = open_serial(SERIAL_PORT1,BAUDRATE_57600);

	#ifdef MB_ADS
	/// Start air data system port and variables *********************
	
	// Creates a buffer to locate the incomming sensor frames 
	uint8_t dataPacketADS[AIR_DATA_SYS_FRAME_SIZE];
	// Starts the air data system
	int portID = airDataSysStart(SERIAL_PORT3, B38400); 
	
	// Sets the minimum rate. the input of function = i=(1,2,3....) i is a multiple of 50ms
	// and represents the delay between data packets
	setADSFrameRate(1);
	// Some dalay 
	cyg_thread_delay(100);
	/// **************************************************************
	
	/// Initialization of Air data system ****************************
	// Sets the altitude to 0 (since the vehicle is static)
	updateADSAltitude(0);
	// Resets the diff. pressure  (since the vehicle is static)
	resetADSDifferentialPressure();
	/// **************************************************************
	#endif

	while ( ofpMode != terminate )
	{
		pthread_mutex_lock (&mutex_imu);
		rc = pthread_cond_wait (&trigger_sensordaq, &mutex_imu);
		
		
		if (rc == 0) {
			
			// compute thread jitter
			toc = get_Time();
			threadstat[0].jitter_nsec = ( (toc-tic) - 1/(double)DAQ_Hz ) * 1E09;
			tic = toc;

			pthread_mutex_lock(&mutex_gps);

			cyg_io_get_config (console_handle, CYG_IO_GET_CONFIG_SERIAL_BUFFER_INFO,
					&buff_info, &len);

			bytesInBuffer = buff_info.rx_count;

			if (bytesInBuffer <= 0){
				//				gpspacket.err_type = gps_nolock;
				//				imupacket.err_type = got_invalid;
			}

			else if(bytesInBuffer > 0) {

				/*********************************************************************
				 * Find start of packet: the header (2 bytes) starts with 0x5555
				 *********************************************************************/
				while (headerOK !=2)
				{
					while(1!=read(sPort1,input_buffer,1));
					if (input_buffer[0] == 0x55)
						headerOK++;
					else
						headerOK = 0;
				}
				headerOK = 0; while(1 != read(sPort1,&input_buffer[2],1));
				nbytes = 3;


				/*********************************************************************
				 *Read packet contents
				 *********************************************************************/
				switch (input_buffer[2])
				{
				case 'S': /* IMU packet without GPS */
					while(nbytes < SENSOR_PACKET_LENGTH)
					{
						nbytes += read(sPort1, input_buffer+nbytes, SENSOR_PACKET_LENGTH-nbytes);
					}

					/*************************
					 * check checksum
					 *************************/
					if(checksum(input_buffer,SENSOR_PACKET_LENGTH)==TRUE)
					{
						decode_imupacket(&imupacket, input_buffer);
						pthread_cond_signal(&trigger_ahrs);
					}
					else imupacket.err_type = checksum_err;

					if (gpsHealthCount++ > 13){
						gpspacket.err_type = gps_nolock;		  // 081607(Rohit): gpspacket.err_type set to false so that GPS loss leads to err_type being set to 'FALSE' or fail state.
						gpsHealthCount = 0;
					}
					break;


				case 'N': /* IMU packet with GPS */

					while(nbytes < FULL_PACKET_SIZE)
					{
						nbytes += read(sPort1, input_buffer+nbytes, FULL_PACKET_SIZE-nbytes);
					}

					/*************************
					 *check checksum
					 *************************/
					if(checksum(input_buffer,FULL_PACKET_SIZE)==TRUE)
					{
						decode_imupacket(&imupacket, input_buffer);
						pthread_cond_signal(&trigger_ahrs);

						/******************************************
						 *check GPS data packet
						 ******************************************/
						if(input_buffer[31]==(byte)0x55 && input_buffer[32]==(byte)0x55 && input_buffer[33]=='G')
						{
							memcpy(&gps_buffer,&input_buffer[31],GPS_PACKET_LENGTH);
							decode_gpspacket(&gpspacket, gps_buffer);
						}
						else gpspacket.err_type = got_invalid;

					} /* end if(checksum(input_buffer... */

					else
					{
						gpspacket.err_type = checksum_err;
						imupacket.err_type = checksum_err;
					}

					gpsHealthCount = 0;

					break;

				} /* end case  */
			}
			
			#ifdef MB_ADS
			/// *****************************************************************************************
			/// Read frame from microbotic air data system 
			// Reads a data frame from the air data system 
			int ans =getAirDataSysFrame(dataPacketADS );
			// If 0, then the data was succesfully  read
			if (ans==0){
			  ///Performs conversions and
			  /// Copy Altitude, airspeed, differential pressure and absolute pressure to imu structure

			  imupacket.h= getADSAltitude(dataPacketADS);  ///  [m]
			  imupacket.ias= getADSAirSpeed(dataPacketADS);  /// [m/s]
			  imupacket.Pd= getADSDifferentialPressure(dataPacketADS);  ///[KPa]
			  imupacket.Ps= getADSAbsolutePressure(dataPacketADS) );  ///[KPa]


			}else{ // Error reading air data sys.
			  
			  //pthread_mutex_lock(&mutex_sPort0);
			  //sprintf(&msg[3],"\n Error in air data system: %d \n" , ans);
			  //write(sPort0, msg, sizeof(msg));
			  //pthread_mutex_unlock(&mutex_sPort0);
			}
			
			/// *****************************************************************************************
			#endif
			
			pthread_mutex_unlock(&mutex_gps);
			pthread_mutex_unlock(&mutex_imu);
		}

	} /* end while */


	//close the serial port
	close(sPort1);
	sprintf(&msg[3],"DAQ thread terminated");
	write(sPort0, msg, sizeof(msg));
	//exit the thread
	pthread_exit(NULL);

} /* end void *imugps_acq() */

/***************************************************************************************
 *check the checksum of the data packet
 ***************************************************************************************/
int checksum(byte* buffer, int packet_len)
{
	word     	 i=0,rcvchecksum=0,tmp;
	unsigned long sum=0;

	for(i=2;i<packet_len-2;i++) sum = sum + buffer[i];
	rcvchecksum = ( (tmp = buffer[packet_len-2] ) << 8) | buffer[packet_len-1];

	if (rcvchecksum == (sum&0xFFFF))
		return    TRUE;
	else
		return    FALSE;
}


/***************************************************************************************
 *decode the gps data packet
 ***************************************************************************************/
void decode_gpspacket(struct gps *data, byte* buffer)
{
	signed long tmp=0;
	

	/* gps velocity in m/s */
	data->vn =(double)((((((tmp = (signed char)buffer[ 6]<<8)|buffer[ 5])<<8)|buffer[ 4])<<8)|buffer[ 3])*1.0e-2; tmp=0;
	data->ve =(double)((((((tmp = (signed char)buffer[10]<<8)|buffer[ 9])<<8)|buffer[ 8])<<8)|buffer[ 7])*1.0e-2; tmp=0;
	data->vd =(double)((((((tmp = (signed char)buffer[14]<<8)|buffer[13])<<8)|buffer[12])<<8)|buffer[11])*1.0e-2; tmp=0;

	/* gps position */
	data->lon=(double)((((((tmp = (signed char)buffer[18]<<8)|buffer[17])<<8)|buffer[16])<<8)|buffer[15])*1.0e-7; tmp=0;
	data->lat=(double)((((((tmp = (signed char)buffer[22]<<8)|buffer[21])<<8)|buffer[20])<<8)|buffer[19])*1.0e-7; tmp=0;
	data->alt=(double)((((((tmp = (signed char)buffer[26]<<8)|buffer[25])<<8)|buffer[24])<<8)|buffer[23])*1.0e-3; tmp=0;

	/* gps time */
	//data->ITOW = (double)((tmp = buffer[28] << 8) | buffer[27]);
	data->GPS_TOW = (unsigned int)((tmp = buffer[28] << 8) | buffer[27]);
	data->err_type = data_valid;
	data->time = get_Time();


}

/***************************************************************************************
 *decode the imu data packet
 ***************************************************************************************/
void decode_imupacket(struct imu *data, byte* buffer)
{
	signed short tmp=0;
	extern double airdata_bias[];

	/* acceleration in m/s^2 */
	data->ax = (double)(((tmp = (signed char)buffer[ 3])<<8)|buffer[ 4])*5.98754883e-04; tmp=0;
	data->ay = (double)(((tmp = (signed char)buffer[ 5])<<8)|buffer[ 6])*5.98754883e-04; tmp=0;
	data->az = (double)(((tmp = (signed char)buffer[ 7])<<8)|buffer[ 8])*5.98754883e-04; tmp=0;


	/* angular rate in rad/s */
	data->p  = (double)(((tmp = (signed char)buffer[ 9])<<8)|buffer[10])*1.065264436e-04; tmp=0;
	data->q  = (double)(((tmp = (signed char)buffer[11])<<8)|buffer[12])*1.065264436e-04; tmp=0;
	data->r  = (double)(((tmp = (signed char)buffer[13])<<8)|buffer[14])*1.065264436e-04; tmp=0;

	/* magnetic field in Gauss */
	data->hx = (double)(((tmp = (signed char)buffer[15])<<8)|buffer[16])*6.103515625e-05; tmp=0;
	data->hy = (double)(((tmp = (signed char)buffer[17])<<8)|buffer[18])*6.103515625e-05; tmp=0;
	data->hz = (double)(((tmp = (signed char)buffer[19])<<8)|buffer[20])*6.103515625e-05; tmp=0;

	/* temperature in Celcius */
	/*
   data->Tx = (double)(((tmp = (signed char)buffer[21])<<8)|buffer[22])*6.103515625e-03; tmp=0;
   data->Ty = (double)(((tmp = (signed char)buffer[23])<<8)|buffer[24])*6.103515625e-03; tmp=0;
   data->Tz = (double)(((tmp = (signed char)buffer[25])<<8)|buffer[26])*6.103515625e-03; tmp=0;
	 */
	
#ifndef MB_ADS
	double ias  = (((tmp = (signed char)buffer[29])<<8)|buffer[30])*2.4414062500e-03; tmp=0;
	data->h = (double)(((tmp = (signed char)buffer[27])<<8)|buffer[28])*3.0517578125e-01-airdata_bias[0]; tmp=0; 
		
	// back out dynamic pressure given indicated airspeed to get pressure bias
	
	data->Pd = (AIR_DATA_P0*( pow( pow(ias/AIR_DATA_K3,2) +1, (1/AIR_DATA_K4))-1 )) ; // [KPa], dynamic pressure given velocity in m/s
	data->ias = copysign(AIR_DATA_K3*sqrt(fabs(pow(fabs((data->Pd-airdata_bias[1])/AIR_DATA_P0 +1),AIR_DATA_K4)-1)),data->Pd);// remove pressure bias and recompute airspeed

#endif


	/*
	// servo packet
	switch (buffer[2]) {
	case 'S' :   servopacket.status = buffer[32];
	servopacket.chn[0] = ((tmpr = buffer[33]) << 8)|buffer[34]; tmpr = 0;
	servopacket.chn[1] = ((tmpr = buffer[35]) << 8)|buffer[36]; tmpr = 0;
	servopacket.chn[2] = ((tmpr = buffer[37]) << 8)|buffer[38]; tmpr = 0;
	servopacket.chn[3] = ((tmpr = buffer[39]) << 8)|buffer[40]; tmpr = 0;
	servopacket.chn[4] = ((tmpr = buffer[41]) << 8)|buffer[42]; tmpr = 0;
	servopacket.chn[5] = ((tmpr = buffer[43]) << 8)|buffer[44]; tmpr = 0;
	servopacket.chn[6] = ((tmpr = buffer[45]) << 8)|buffer[46]; tmpr = 0;
	servopacket.chn[7] = ((tmpr = buffer[47]) << 8)|buffer[48]; tmpr = 0;
	break;
	case 'N' :   servopacket.status = buffer[67];
	servopacket.chn[0] = ((tmpr = buffer[68]) << 8)|buffer[69]; tmpr = 0;
	servopacket.chn[1] = ((tmpr = buffer[70]) << 8)|buffer[71]; tmpr = 0;
	servopacket.chn[2] = ((tmpr = buffer[72]) << 8)|buffer[73]; tmpr = 0;
	servopacket.chn[3] = ((tmpr = buffer[74]) << 8)|buffer[75]; tmpr = 0;
	servopacket.chn[4] = ((tmpr = buffer[76]) << 8)|buffer[77]; tmpr = 0;
	servopacket.chn[5] = ((tmpr = buffer[78]) << 8)|buffer[79]; tmpr = 0;
	servopacket.chn[6] = ((tmpr = buffer[80]) << 8)|buffer[81]; tmpr = 0;
	servopacket.chn[7] = ((tmpr = buffer[82]) << 8)|buffer[83]; tmpr = 0;
	break;
	default  :
		//                   printf("[imu]:fail to decode servo packet..!\n");
		break;
	}
	 */
	data->time = get_Time();
	data->err_type = data_valid;

}
